@@ -11,7 +11,7 @@ use error::{ShapeError, ErrorKind, from_kind};
 
 /// Stack arrays along the given axis.
 ///
-/// ***Errors*** if the arrays don't all have the same dimension
+/// ***Errors*** if the arrays have mismatching shapes, apart from along `axis`.
 /// (may be made more flexible in the future).<br>
 /// ***Errors*** if `arrays` is empty, if `axis` is out of bounds,
 /// if the result is larger than is possible to represent.
@@ -57,7 +57,7 @@ pub fn stack<'a, A, D>(axis: Axis, arrays: &[ArrayView<'a, A, D>])
     unsafe {
         v.set_len(size);
     }
-    let mut res = try!(OwnedArray::from_vec_dim(res_dim, v));
+    let mut res = try!(OwnedArray::from_shape_vec(res_dim, v));
 
     {
         let mut assign_view = res.view_mut();
@@ -102,6 +102,6 @@ pub fn stack<'a, A, D>(axis: Axis, arrays: &[ArrayView<'a, A, D>])
 #[macro_export]
 macro_rules! stack {
     ($axis:expr, $( $array:expr ),+ ) => {
-        ndarray::stack($axis, &[ $(ndarray::ArrayView::from(&$array) ),* ]).unwrap()
+        $crate::stack($axis, &[ $($crate::ArrayView::from(&$array) ),* ]).unwrap()
     }
 }
