@@ -4,20 +4,15 @@
 extern crate ndarray;
 extern crate itertools;
 
-use ndarray::{RcArray, S, Si,
-    OwnedArray,
-};
+use ndarray::{S, Si};
+use ndarray::prelude::*;
 use ndarray::{
     rcarr2,
-    arr0, arr1, arr2, arr3,
-    aview0,
-    aview1,
-    aview2,
+    arr0, arr3,
     aview_mut1,
-    Dimension,
 };
 use ndarray::Indexes;
-use ndarray::Axis;
+use itertools::free::enumerate;
 
 #[test]
 fn test_matmul_rcarray()
@@ -32,12 +27,12 @@ fn test_matmul_rcarray()
         *elt = i;
     }
 
-    let c = A.mat_mul(&B);
+    let c = A.dot(&B);
     println!("A = \n{:?}", A);
     println!("B = \n{:?}", B);
     println!("A x B = \n{:?}", c);
     unsafe {
-        let result = RcArray::from_vec_dim_unchecked((2, 4), vec![20, 23, 26, 29, 56, 68, 80, 92]);
+        let result = RcArray::from_shape_vec_unchecked((2, 4), vec![20, 23, 26, 29, 56, 68, 80, 92]);
         assert_eq!(c.shape(), result.shape());
         assert!(c.iter().zip(result.iter()).all(|(a,b)| a == b));
         assert!(c == result);
@@ -50,9 +45,9 @@ fn test_mat_mul() {
     let (n, m) = (45, 33);
     let a = RcArray::linspace(0., ((n * m) - 1) as f32, n as usize * m as usize ).reshape((n, m));
     let b = RcArray::eye(m);
-    assert_eq!(a.mat_mul(&b), a);
+    assert_eq!(a.dot(&b), a);
     let c = RcArray::eye(n);
-    assert_eq!(c.mat_mul(&a), a);
+    assert_eq!(c.dot(&a), a);
 }
 
 
@@ -221,6 +216,33 @@ fn test_sub()
     assert_eq!(m, mat.subview(Axis(1), 1));
 }
 
+
+#[test]
+fn test_select(){
+    // test for 2-d array
+    let x = arr2(&[[0., 1.], [1.,0.],[1.,0.],[1.,0.],[1.,0.],[0., 1.],[0., 1.]]);
+    let r = x.select(Axis(0),&[1,3,5]);
+    let c = x.select(Axis(1),&[1]);
+    let r_target = arr2(&[[1.,0.],[1.,0.],[0., 1.]]);
+    let c_target = arr2(&[[1.,0.,0.,0.,0., 1., 1.]]);
+    assert!(r.all_close(&r_target,1e-8));
+    assert!(c.all_close(&c_target.t(),1e-8));
+
+    // test for 3-d array
+    let y = arr3(&[[[1., 2., 3.],
+                    [1.5, 1.5, 3.]],
+                    [[1., 2., 8.],
+                    [1., 2.5, 3.]]]);
+    let r = y.select(Axis(1),&[1]);
+    let c = y.select(Axis(2),&[1]);
+    let r_target = arr3(&[[[1.5, 1.5, 3.]], [[1., 2.5, 3.]]]);
+    let c_target = arr3(&[[[2.],[1.5]],[[2.],[2.5]]]);
+    assert!(r.all_close(&r_target,1e-8));
+    assert!(c.all_close(&c_target,1e-8));
+
+}
+
+
 #[test]
 fn diag()
 {
@@ -426,7 +448,7 @@ fn owned_array_with_stride() {
     let dim = (2, 3, 2);
     let strides = (1, 4, 2);
 
-    let a = OwnedArray::from_vec_dim_stride(dim, strides, v).unwrap();
+    let a = OwnedArray::from_shape_vec(dim.strides(strides), v).unwrap();
     assert_eq!(a.strides(), &[1, 4, 2]);
 }
 
@@ -443,7 +465,7 @@ macro_rules! assert_matches {
 #[test]
 fn from_vec_dim_stride_empty_1d() {
     let empty: [f32; 0] = [];
-    assert_matches!(OwnedArray::from_vec_dim_stride(0, 1, empty.to_vec()),
+    assert_matches!(OwnedArray::from_shape_vec(0.strides(1), empty.to_vec()),
                     Ok(_));
 }
 
@@ -453,11 +475,11 @@ fn from_vec_dim_stride_0d() {
     let one = [1.];
     let two = [1., 2.];
     // too few elements
-    assert_matches!(OwnedArray::from_vec_dim_stride((), (), empty.to_vec()), Err(_));
+    assert_matches!(OwnedArray::from_shape_vec(().strides(()), empty.to_vec()), Err(_));
     // exact number of elements
-    assert_matches!(OwnedArray::from_vec_dim_stride((), (), one.to_vec()), Ok(_));
+    assert_matches!(OwnedArray::from_shape_vec(().strides(()), one.to_vec()), Ok(_));
     // too many are ok
-    assert_matches!(OwnedArray::from_vec_dim_stride((), (), two.to_vec()), Ok(_));
+    assert_matches!(OwnedArray::from_shape_vec(().strides(()), two.to_vec()), Ok(_));
 }
 
 #[test]
@@ -465,7 +487,7 @@ fn from_vec_dim_stride_2d_1() {
     let two = [1., 2.];
     let d = (2, 1);
     let s = d.default_strides();
-    assert_matches!(OwnedArray::from_vec_dim_stride(d, s, two.to_vec()), Ok(_));
+    assert_matches!(OwnedArray::from_shape_vec(d.strides(s), two.to_vec()), Ok(_));
 }
 
 #[test]
@@ -473,7 +495,7 @@ fn from_vec_dim_stride_2d_2() {
     let two = [1., 2.];
     let d = (1, 2);
     let s = d.default_strides();
-    assert_matches!(OwnedArray::from_vec_dim_stride(d, s, two.to_vec()), Ok(_));
+    assert_matches!(OwnedArray::from_shape_vec(d.strides(s), two.to_vec()), Ok(_));
 }
 
 #[test]
@@ -483,7 +505,7 @@ fn from_vec_dim_stride_2d_3() {
                    [[3]]]);
     let d = a.dim();
     let s = d.default_strides();
-    assert_matches!(OwnedArray::from_vec_dim_stride(d, s, a.as_slice().unwrap().to_vec()), Ok(_));
+    assert_matches!(OwnedArray::from_shape_vec(d.strides(s), a.as_slice().unwrap().to_vec()), Ok(_));
 }
 
 #[test]
@@ -493,7 +515,7 @@ fn from_vec_dim_stride_2d_4() {
                    [[3]]]);
     let d = a.dim();
     let s = d.fortran_strides();
-    assert_matches!(OwnedArray::from_vec_dim_stride(d, s, a.as_slice().unwrap().to_vec()), Ok(_));
+    assert_matches!(OwnedArray::from_shape_vec(d.strides(s), a.as_slice().unwrap().to_vec()), Ok(_));
 }
 
 #[test]
@@ -501,7 +523,7 @@ fn from_vec_dim_stride_2d_5() {
     let a = arr3(&[[[1, 2, 3]]]);
     let d = a.dim();
     let s = d.fortran_strides();
-    assert_matches!(OwnedArray::from_vec_dim_stride(d, s, a.as_slice().unwrap().to_vec()), Ok(_));
+    assert_matches!(OwnedArray::from_shape_vec(d.strides(s), a.as_slice().unwrap().to_vec()), Ok(_));
 }
 
 #[test]
@@ -509,11 +531,11 @@ fn from_vec_dim_stride_2d_6() {
     let a = [1., 2., 3., 4., 5., 6.];
     let d = (2, 1, 1);
     let s = (2, 2, 1);
-    assert_matches!(OwnedArray::from_vec_dim_stride(d, s, a.to_vec()), Ok(_));
+    assert_matches!(OwnedArray::from_shape_vec(d.strides(s), a.to_vec()), Ok(_));
 
     let d = (1, 2, 1);
     let s = (2, 2, 1);
-    assert_matches!(OwnedArray::from_vec_dim_stride(d, s, a.to_vec()), Ok(_));
+    assert_matches!(OwnedArray::from_shape_vec(d.strides(s), a.to_vec()), Ok(_));
 }
 
 #[test]
@@ -523,7 +545,7 @@ fn from_vec_dim_stride_2d_7() {
     // [[]] shape=[4, 0], strides=[0, 1]
     let d = (4, 0);
     let s = (0, 1);
-    assert_matches!(OwnedArray::from_vec_dim_stride(d, s, a.to_vec()), Ok(_));
+    assert_matches!(OwnedArray::from_shape_vec(d.strides(s), a.to_vec()), Ok(_));
 }
 
 #[test]
@@ -532,7 +554,7 @@ fn from_vec_dim_stride_2d_8() {
     let a = [1.];
     let d = (1, 1);
     let s = (0, 1);
-    assert_matches!(OwnedArray::from_vec_dim_stride(d, s, a.to_vec()), Err(_));
+    assert_matches!(OwnedArray::from_shape_vec(d.strides(s), a.to_vec()), Err(_));
 }
 
 #[test]
@@ -540,11 +562,11 @@ fn from_vec_dim_stride_2d_rejects() {
     let two = [1., 2.];
     let d = (2, 2);
     let s = (1, 0);
-    assert_matches!(OwnedArray::from_vec_dim_stride(d, s, two.to_vec()), Err(_));
+    assert_matches!(OwnedArray::from_shape_vec(d.strides(s), two.to_vec()), Err(_));
 
     let d = (2, 2);
     let s = (0, 1);
-    assert_matches!(OwnedArray::from_vec_dim_stride(d, s, two.to_vec()), Err(_));
+    assert_matches!(OwnedArray::from_shape_vec(d.strides(s), two.to_vec()), Err(_));
 }
 
 #[test]
@@ -606,7 +628,6 @@ fn slice_mut() {
                            [99, 5, 99]]));
 }
 
-#[cfg(feature = "assign_ops")]
 #[test]
 fn assign_ops()
 {
@@ -640,9 +661,10 @@ fn aview_mut() {
     let mut data = [0; 16];
     {
         let mut a = aview_mut1(&mut data).into_shape((4, 4)).unwrap();
-        a.slice_mut(&[Si(0, Some(2), 1), Si(0, None, 2)])
-         .iadd_scalar(&1);
-        println!("{}", a);
+        {
+            let mut slc = a.slice_mut(s![..2, ..;2]);
+            slc += 1;
+        }
     }
     assert_eq!(data, [1, 0, 1, 0,  1, 0, 1, 0,  0, 0, 0, 0,  0, 0, 0, 0]);
 }
@@ -706,11 +728,39 @@ fn reshape_error1() {
 fn reshape_error2() {
     let data = [1, 2, 3, 4, 5, 6, 7, 8];
     let v = aview1(&data);
-    let mut u = v.into_shape((4, 2)).unwrap();
+    let mut u = v.into_shape((2, 2, 2)).unwrap();
     u.swap_axes(0, 1);
     let _s = u.into_shape((2, 4)).unwrap();
 }
 
+#[test]
+fn reshape_f() {
+    let mut u = OwnedArray::zeros((3, 4).f());
+    for (i, elt) in enumerate(u.as_slice_memory_order_mut().unwrap()) {
+        *elt = i as i32;
+    }
+    let v = u.view();
+    println!("{:?}", v);
+
+    // noop ok
+    let v2 = v.into_shape((3, 4));
+    assert!(v2.is_ok());
+    assert_eq!(v, v2.unwrap());
+
+    let u = v.into_shape((3, 2, 2));
+    assert!(u.is_ok());
+    let u = u.unwrap();
+    println!("{:?}", u);
+    assert_eq!(u.shape(), &[3, 2, 2]);
+    let s = u.into_shape((4, 3)).unwrap();
+    println!("{:?}", s);
+    assert_eq!(s.shape(), &[4, 3]);
+    assert_eq!(s, aview2(&[[0, 4, 8],
+                           [1, 5, 9],
+                           [2, 6,10],
+                           [3, 7,11]]));
+}
+
 #[test]
 fn arithmetic_broadcast() {
     let mut a = arr2(&[[1., 2.], [3., 4.]]);
@@ -769,9 +819,9 @@ fn scalar_ops() {
 #[test]
 fn deny_wraparound_from_vec() {
     let five = vec![0; 5];
-    let _five_large = OwnedArray::from_vec_dim((3, 7, 29, 36760123, 823996703), five.clone());
-    assert!(_five_large.is_err());
-    let six = OwnedArray::from_vec_dim(6, five.clone());
+    let five_large = OwnedArray::from_shape_vec((3, 7, 29, 36760123, 823996703), five.clone());
+    assert!(five_large.is_err());
+    let six = OwnedArray::from_shape_vec(6, five.clone());
     assert!(six.is_err());
 }
 
@@ -868,7 +918,7 @@ fn test_f_order() {
     // even if the underlying memory order is different
     let c = arr2(&[[1, 2, 3],
                    [4, 5, 6]]);
-    let mut f = OwnedArray::zeros_f(c.dim());
+    let mut f = OwnedArray::zeros(c.dim().f());
     f.assign(&c);
     assert_eq!(f, c);
     assert_eq!(f.shape(), c.shape());
@@ -935,7 +985,7 @@ fn test_contiguous() {
     assert!(v.as_slice_memory_order().is_some());
 
     let a = OwnedArray::<f32, _>::zeros((20, 1));
-    let b = OwnedArray::<f32, _>::zeros_f((20, 1));
+    let b = OwnedArray::<f32, _>::zeros((20, 1).f());
     assert!(a.as_slice().is_some());
     assert!(b.as_slice().is_some());
     assert!(a.as_slice_memory_order().is_some());
@@ -957,3 +1007,32 @@ fn test_all_close() {
     assert!(c.all_close(&aview1(&[1., 2., 3.]), 1.));
     assert!(!c.all_close(&aview1(&[1., 2., 3.]), 0.1));
 }
+
+#[test]
+fn test_swap() {
+    let mut a = arr2(&[[1, 2, 3],
+                       [4, 5, 6],
+                       [7, 8, 9]]);
+    let b = a.clone();
+
+    for i in 0..a.rows() {
+        for j in i + 1..a.cols() {
+            a.swap((i, j), (j, i));
+        }
+    }
+    assert_eq!(a, b.t());
+}
+
+#[test]
+fn test_shape() {
+    let data = [0, 1, 2, 3, 4, 5];
+    let a = OwnedArray::from_shape_vec((1, 2, 3), data.to_vec()).unwrap();
+    let b = OwnedArray::from_shape_vec((1, 2, 3).f(), data.to_vec()).unwrap();
+    let c = OwnedArray::from_shape_vec((1, 2, 3).strides((1, 3, 1)), data.to_vec()).unwrap();
+    println!("{:?}", a);
+    println!("{:?}", b);
+    println!("{:?}", c);
+    assert_eq!(a.strides(), &[6, 3, 1]);
+    assert_eq!(b.strides(), &[1, 1, 2]);
+    assert_eq!(c.strides(), &[1, 3, 1]);
+}
