@@ -1,10 +1,12 @@
 
-extern crate num;
+extern crate num_traits;
+extern crate num_complex;
 extern crate ndarray;
 
 use ndarray::{arr1, arr2, Axis};
 use ndarray::OwnedArray;
-use num::{Num, Complex};
+use num_traits::Num;
+use num_complex::Complex;
 
 fn c<T: Clone + Num>(re: T, im: T) -> Complex<T> {
     Complex::new(re, im)
@@ -17,7 +19,7 @@ fn complex_mat_mul()
     let b = (&a * c(3., 0.)).map(|c| 5. * c / c.norm());
     println!("{:>8.2}", b);
     let e = OwnedArray::eye(2);
-    let r = a.mat_mul(&e);
+    let r = a.dot(&e);
     println!("{}", a);
     assert_eq!(r, a);
     assert_eq!(a.mean(Axis(0)), arr1(&[c(1.5, 1.), c(2.5, 0.)]));
